package com.compomics.marc;

import com.compomics.util.io.export.styles.DefaultStyle;
import com.compomics.util.io.export.writers.ExcelWriter;
import com.compomics.util.math.roc.DataRoc;
import com.compomics.util.math.roc.DistributionRoc;
import com.compomics.util.math.statistics.ROC;
import com.compomics.util.math.statistics.distributions.NonSymmetricalNormalDistribution;
import it.unimi.dsi.fastutil.Hash;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import org.apache.commons.math.MathException;

/**
 *
 * @author Marc
 */
public class RocScript {

    private static final String separator = "\t";

    public static void main(String[] args) {
        RocScript m = new RocScript();

        try {
            m.yngvildRoc();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void yngvildRoc() throws FileNotFoundException, IOException, MathException {

        // Input
        File inFile = new File("D:\\projects\\biomarkers Yngvild\\input_roc.txt");

        HashMap<String, ArrayList<Double>> controlInput = new HashMap<String, ArrayList<Double>>(),
                healthyMutInput = new HashMap<String, ArrayList<Double>>(),
                diabetesInput = new HashMap<String, ArrayList<Double>>();
        HashMap<String, NonSymmetricalNormalDistribution> controlDistributions = new HashMap<String, NonSymmetricalNormalDistribution>(),
                healthyMutDistributions = new HashMap<String, NonSymmetricalNormalDistribution>(),
                diabetesMutationDistributions = new HashMap<String, NonSymmetricalNormalDistribution>();
        HashMap<String, ROC> mutationDistributionRocs = new HashMap<String, ROC>(),
                mutationDataRocs = new HashMap<String, ROC>(),
                diabetesDistributionRocs = new HashMap<String, ROC>(),
                diabetesDataRocs = new HashMap<String, ROC>();
        ArrayList<String> transitions = new ArrayList<String>();
        HashMap<String, Double> lods = new HashMap<String, Double>();
        HashMap<String, Integer> ns = new HashMap<String, Integer>();
        HashMap<String, Integer> nsNotNull = new HashMap<String, Integer>();

        BufferedReader br = new BufferedReader(new FileReader(inFile));
        try {
            ArrayList<StringBuilder> transitionsTemp = new ArrayList<StringBuilder>();
            String line = br.readLine();
            String[] split = line.split(separator);
            for (int cpt = 1; cpt < split.length; cpt++) {
                transitionsTemp.add(new StringBuilder(split[cpt]));
            }
            line = br.readLine();
            split = line.split(separator);
            for (int cpt = 1; cpt < split.length; cpt++) {
                String peptide = split[cpt];
                peptide = peptide.trim();
                if (peptide.length() > 0) {
                    StringBuilder transition = transitionsTemp.get(cpt - 1);
                    transition.append(" ").append(peptide);
                }
            }
            line = br.readLine();
            split = line.split(separator);
            for (int cpt = 1; cpt < split.length; cpt++) {
                String ion = split[cpt];
                ion = ion.trim();
                if (ion.length() > 0) {
                    StringBuilder transition = transitionsTemp.get(cpt - 1);
                    transition.append(" ").append(ion);
                }
            }
            for (StringBuilder transition : transitionsTemp) {
                transitions.add(transition.toString());
            }

            while ((line = br.readLine()) != null) {
                line = line.trim();
                if (!line.equals("")) {
                    split = line.split(separator);
                    Integer patientClass = new Integer(split[0]);
                    for (int cpt = 1; cpt < split.length; cpt++) {
                        String transition = transitions.get(cpt - 1);
                        ArrayList<Double> data;
                        if (patientClass == 0) {
                            data = controlInput.get(transition);
                            if (data == null) {
                                data = new ArrayList<Double>();
                                controlInput.put(transition, data);
                            }
                        } else if (patientClass == 1) {
                            data = healthyMutInput.get(transition);
                            if (data == null) {
                                data = new ArrayList<Double>();
                                healthyMutInput.put(transition, data);
                            }
                        } else {
                            data = diabetesInput.get(transition);
                            if (data == null) {
                                data = new ArrayList<Double>();
                                diabetesInput.put(transition, data);
                            }
                        }
                        try {
                            Double value = new Double(split[cpt]);
                            data.add(value);

                            // LOD
                            if (value > 0) {
                                Double lod = lods.get(transition);
                                if (lod == null || lod > value) {
                                    lods.put(transition, value);
                                }
                            }

                            // n
                            Integer n = ns.get(transition);
                            if (n == null) {
                                ns.put(transition, 1);
                            } else {
                                ns.put(transition, n + 1);
                            }

                            // n>0
                            if (value > 0) {
                                n = nsNotNull.get(transition);
                                if (n == null) {
                                    nsNotNull.put(transition, 1);
                                } else {
                                    nsNotNull.put(transition, n + 1);
                                }
                            }

                        } catch (Exception e) {
                            // not a number
                        }
                    }
                }
            }
        } finally {
            br.close();
        }

        // Distributions & ROCS
        for (String transition : transitions) {
            ArrayList<Double> controlValues = controlInput.get(transition);
            NonSymmetricalNormalDistribution controlDistribution = null,
                    healthyMutDistribution = null,
                    diabetesDistribution = null;
            if (controlValues != null && !controlValues.isEmpty()) {
                controlDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(controlValues);
                controlDistributions.put(transition, controlDistribution);
            }
            ArrayList<Double> healthyMutValues = healthyMutInput.get(transition);
            if (healthyMutValues != null && !healthyMutValues.isEmpty()) {
                healthyMutDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(healthyMutValues);
                healthyMutDistributions.put(transition, healthyMutDistribution);
            }
            ArrayList<Double> diabetesValues = diabetesInput.get(transition);
            if (diabetesValues != null && !diabetesValues.isEmpty()) {
                diabetesDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(diabetesValues);
                diabetesMutationDistributions.put(transition, diabetesDistribution);
            }

            if (controlDistribution != null && healthyMutDistribution != null) {
                DistributionRoc distributionRoc = new DistributionRoc(controlDistribution, healthyMutDistribution);
                mutationDistributionRocs.put(transition, distributionRoc);
                DataRoc dataRoc = new DataRoc(controlValues, healthyMutValues, DataRoc.RocInterpolation.minimum);
                mutationDataRocs.put(transition, dataRoc);
            }
            if (healthyMutDistribution != null && diabetesDistribution != null) {
                DistributionRoc distributionRoc = new DistributionRoc(healthyMutDistribution, diabetesDistribution);
                diabetesDistributionRocs.put(transition, distributionRoc);
                DataRoc dataRoc = new DataRoc(healthyMutValues, diabetesValues, DataRoc.RocInterpolation.minimum);
                diabetesDataRocs.put(transition, dataRoc);
            }
        }

        // output
        File outFile = new File("D:\\projects\\biomarkers Yngvild\\30.09.14_rocs.xls");
        ExcelWriter ew = new ExcelWriter(outFile);
        DefaultStyle style = DefaultStyle.getStyle(ew);
        ew.setWorkbookStyle(style);

        ew.startNewSection("Summary");

        ew.writeHeaderText("Transition");
        ew.addSeparator();
        ew.writeHeaderText("N");
        ew.addSeparator();
        ew.writeHeaderText("N>0");
        ew.addSeparator();
        ew.writeHeaderText("LOD");
        ew.addSeparator();
        ew.writeHeaderText("Healthy Mut Vs Control - AUC (Empirical)");
        ew.addSeparator();
        ew.writeHeaderText("Healthy Mut Vs Control - AUC (Statistical)");
        ew.addSeparator();
        ew.writeHeaderText("Diabetes Vs Healthy Mut - AUC (Empirical)");
        ew.addSeparator();
        ew.writeHeaderText("Diabetes Vs Healthy Mut - AUC (Statistical)");
        ew.newLine();
        for (String transition : transitions) {
            ew.writeHeaderText(transition);
            ew.addSeparator();
            ew.write(ns.get(transition) + "");
            ew.addSeparator();
            ew.write(nsNotNull.get(transition) + "");
            ew.addSeparator();
            ew.write(lods.get(transition) + "");
            ew.addSeparator();
            ROC roc = mutationDataRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            roc = mutationDistributionRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            roc = diabetesDataRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            roc = diabetesDistributionRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.newLine();
        }

        ew.startNewSection("Mutation Healthy Vs Control");

        for (String transition : transitions) {
            ew.writeHeaderText(transition);
            ew.addSeparator();
            ew.addSeparator();
            ew.addSeparator();
            ew.addSeparator();
        }
        ew.newLine();

        // ROC titles
        for (String transition : transitions) {
            ew.writeHeaderText("ROC (Empirical)");
            ew.addSeparator();
            ew.addSeparator();
            ew.writeHeaderText("Statistical ROC (Statistical)");
            ew.addSeparator();
            ew.addSeparator();
        }
        ew.newLine();

        // AUC
        for (String transition : transitions) {
            ew.write("AUC");
            ew.addSeparator();
            ROC roc = mutationDataRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            ew.write("AUC");
            ew.addSeparator();
            roc = mutationDistributionRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
        }
        ew.newLine();

        // ROC titles
        for (String transition : transitions) {
            ew.write("1-specificity");
            ew.addSeparator();
            ew.write("sensitivity");
            ew.addSeparator();
            ew.write("1-specificity");
            ew.addSeparator();
            ew.write("sensitivity");
            ew.addSeparator();
        }
        ew.newLine();

        // AUC
        int cpt = 0;
        boolean allDone = false;
        while (!allDone) {
            allDone = true;
            for (String transition : transitions) {
                ROC roc = mutationDataRocs.get(transition);
                if (roc != null) {
                    double[][] xy = roc.getxYValues();
                    if (cpt < xy.length) {
                        allDone = false;
                        ew.write(xy[cpt][0] + "");
                        ew.addSeparator();
                        ew.write(xy[cpt][1] + "");
                        ew.addSeparator();
                    } else {
                        ew.addSeparator();
                        ew.addSeparator();
                    }
                } else {
                    ew.addSeparator();
                    ew.addSeparator();
                }
                roc = mutationDistributionRocs.get(transition);
                if (roc != null) {
                    double[][] xy = roc.getxYValues();
                    if (cpt < xy.length) {
                        allDone = false;
                        ew.write(xy[cpt][0] + "");
                        ew.addSeparator();
                        ew.write(xy[cpt][1] + "");
                        ew.addSeparator();
                    } else {
                        ew.addSeparator();
                        ew.addSeparator();
                    }
                } else {
                    ew.addSeparator();
                    ew.addSeparator();
                }
            }
            ew.newLine();
            cpt++;
        }
        ew.newLine();

        // Comparison
        ew.startNewSection("Diabetes Vs Mutation Healthy");

        for (String transition : transitions) {
            ew.writeHeaderText(transition);
            ew.addSeparator();
            ew.addSeparator();
            ew.addSeparator();
            ew.addSeparator();
        }
        ew.newLine();

        // ROC titles
        for (String transition : transitions) {
            ew.writeHeaderText("Empirical ROC");
            ew.addSeparator();
            ew.addSeparator();
            ew.writeHeaderText("Statistical ROC");
            ew.addSeparator();
            ew.addSeparator();
        }
        ew.newLine();

        // AUC
        for (String transition : transitions) {
            ew.write("AUC");
            ew.addSeparator();
            ROC roc = diabetesDataRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
            ew.write("AUC");
            ew.addSeparator();
            roc = diabetesDistributionRocs.get(transition);
            if (roc != null) {
                ew.write(roc.getAuc() + "");
            }
            ew.addSeparator();
        }
        ew.newLine();

        // ROC titles
        for (String transition : transitions) {
            ew.write("1-specificity");
            ew.addSeparator();
            ew.write("sensitivity");
            ew.addSeparator();
            ew.write("1-specificity");
            ew.addSeparator();
            ew.write("sensitivity");
            ew.addSeparator();
        }
        ew.newLine();

        // AUC
        cpt = 0;
        allDone = false;
        while (!allDone) {
            allDone = true;
            for (String transition : transitions) {
                ROC roc = diabetesDataRocs.get(transition);
                if (roc != null) {
                    double[][] xy = roc.getxYValues();
                    if (cpt < xy.length) {
                        allDone = false;
                        ew.write(xy[cpt][0] + "");
                        ew.addSeparator();
                        ew.write(xy[cpt][1] + "");
                        ew.addSeparator();
                    } else {
                        ew.addSeparator();
                        ew.addSeparator();
                    }
                } else {
                    ew.addSeparator();
                    ew.addSeparator();
                }
                roc = diabetesDistributionRocs.get(transition);
                if (roc != null) {
                    double[][] xy = roc.getxYValues();
                    if (cpt < xy.length) {
                        allDone = false;
                        ew.write(xy[cpt][0] + "");
                        ew.addSeparator();
                        ew.write(xy[cpt][1] + "");
                        ew.addSeparator();
                    } else {
                        ew.addSeparator();
                        ew.addSeparator();
                    }
                } else {
                    ew.addSeparator();
                    ew.addSeparator();
                }
            }
            ew.newLine();
            cpt++;
        }

        ew.close();

    }

    public void patientControlRoc() throws FileNotFoundException, IOException, MathException {

        // Input
        File inFile = new File("D:\\projects\\biomarkers Yngvild\\input_roc.txt");

        HashMap<String, ArrayList<Double>> controlInput = new HashMap<String, ArrayList<Double>>(),
                msInput = new HashMap<String, ArrayList<Double>>();
        HashMap<String, NonSymmetricalNormalDistribution> controlDistributions = new HashMap<String, NonSymmetricalNormalDistribution>(),
                msDistributions = new HashMap<String, NonSymmetricalNormalDistribution>();
        HashMap<String, ROC> rocs = new HashMap<String, ROC>();
        ArrayList<String> proteins = new ArrayList<String>();

        BufferedReader br = new BufferedReader(new FileReader(inFile));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String[] split = line.split(separator);
                String accession = split[0];
                proteins.add(accession);
                ArrayList<Double> input = new ArrayList<Double>();
                for (int i = 5; i <= 68; i++) {
                    String intensity = split[i];
                    if (!intensity.equals("")) {
                        input.add(new Double(intensity));
                    }
                }
                controlInput.put(accession, input);
                NonSymmetricalNormalDistribution controlDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(input);
                controlDistributions.put(accession, controlDistribution);
                input = new ArrayList<Double>();
                for (int i = 69; i <= 105; i++) {
                    String intensity = split[i];
                    if (!intensity.equals("")) {
                        input.add(new Double(intensity));
                    }
                }
                msInput.put(accession, input);
                NonSymmetricalNormalDistribution msDistribution = NonSymmetricalNormalDistribution.getRobustNonSymmetricalNormalDistribution(input);
                msDistributions.put(accession, msDistribution);

                DistributionRoc roc = new DistributionRoc(controlDistribution, msDistribution);
                rocs.put(accession, roc);
            }
        } finally {
            br.close();
        }

        // output
        File outFile = new File("D:\\projects\\jill roc\\rocs.txt");
        BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
        for (String accession : proteins) {
            bw.write(separator + accession);
        }
        bw.newLine();

        try {
            for (double x = 0; x <= 1; x += 0.01) {
                bw.write(x + "");
                for (String accession : proteins) {
                    ROC roc = rocs.get(accession);
                    double y = roc.getValueAt(x);
                    bw.write(separator + y);
                }
                bw.newLine();
            }
        } finally {
            bw.close();
        }

        // inverted output
        outFile = new File("D:\\projects\\jill roc\\inverted_rocs.txt");
        bw = new BufferedWriter(new FileWriter(outFile));
        for (String accession : proteins) {
            bw.write(separator + accession);
        }
        bw.newLine();

        try {
            for (double x = 0; x <= 1; x += 0.01) {
                bw.write(x + "");
                for (String accession : proteins) {
                    ROC roc = rocs.get(accession);
                    double y = roc.getSpecificityAt(x);
                    bw.write(separator + y);
                }
                bw.newLine();
            }
        } finally {
            bw.close();
        }
    }

}
